@@ -56,7 +56,7 @@ impl InlineString {
         InlineString { buffer }
     }
 
-    /// Creates an `InlineString` from raw parts without checking that it's valid UTF-8
+    /// Creates an [`InlineString`] from raw parts without checking that it's valid UTF-8
     #[inline]
     pub const unsafe fn from_parts(len: usize, mut buffer: [u8; MAX_SIZE]) -> Self {
         if len != MAX_SIZE {
@@ -89,7 +89,7 @@ impl InlineString {
 
     /// Provides a mutable reference to the underlying buffer
     ///
-    /// # Invariants
+    /// # Safety
     /// * Please see `super::Repr` for all invariants
     #[inline]
     pub unsafe fn as_mut_slice(&mut self) -> &mut [u8] {
@@ -116,12 +116,13 @@ mod tests {
     use std::convert::TryFrom;
 
     use proptest::prelude::*;
+    use test_strategy::proptest;
 
     use super::{
         InlineString,
         MAX_SIZE,
     };
-    use crate::tests::rand_unicode_bytes;
+    use crate::tests::rand_unicode_with_max_len;
 
     #[test]
     fn test_sanity() {
@@ -133,15 +134,13 @@ mod tests {
         assert_eq!(inline.capacity(), MAX_SIZE);
     }
 
-    proptest! {
-        #[test]
-        #[cfg_attr(miri, ignore)]
-        fn test_roundtrip(s in rand_unicode_bytes(MAX_SIZE)) {
-            let inline = InlineString::new(&s);
+    #[proptest]
+    #[cfg_attr(miri, ignore)]
+    fn test_roundtrip(#[strategy(rand_unicode_with_max_len(MAX_SIZE))] s: String) {
+        let inline = InlineString::new(&s);
 
-            assert_eq!(inline.as_str(), s);
-            assert_eq!(inline.len(), s.len());
-        }
+        prop_assert_eq!(inline.len(), s.len());
+        prop_assert_eq!(inline.as_str(), s);
     }
 
     #[test]
